%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Main Menu
% 31 January 2006, Release 1.3
%
% Input:
%
% Output:
%
% Internal Variables:
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all; close all; clc

% blow the train whistle 
load xtrain
p=audioplayer(y, Fs);
play(p)

% echo out the disclaimers
disp('***************************************************************************************************')
disp('*** Merged version 1.3 of channel model MATLAB code (TSV Engine and SV engine) Jan. 31 2007     ***')
disp('*** Programmed by Richard D Roberts (SV engine), Hiroshi Harada, Ryuhei Funada, Hirokazu Sawada ***')
disp('*** Chan-Soon Choi, Yozo Shoji and Shuzo Kato (TSV engine)                                      ***')
disp('***************************************************************************************************')
disp(' ')
disp('***************')
disp('*** History ***')
disp('***************')
disp(' ')
disp('*** SV engine ***')
disp(' ')
disp('*** Version Release 1.100, January 31 2006 ***')
disp(' ')
disp('*** TSV engine ***')
disp(' ')
disp('*** Version Release 1.300, March 1 2007 ***')
disp(' ')
disp('---- Feature ----')
disp('  1. Supported all the CM (form CM1 to CM9)')
disp('  2. Generated continuous data and resampled data')
disp('  3. Included reference antenna pattern discussed in Nov. 2006')
disp('  4. Implemented all of the changes discussed in Jan. 2007')
disp(' ')
disp('---- Bug report ----')
disp(' Jan 9, 2007 rev. 1.01 - added to Menu, SV models CM1.5, CM2.5 and CM9.3')
disp(' ')

% example of Menu Merge

tsv_or_sv=input('Do you want to run TSV (1) or SV (2) model? ');

if tsv_or_sv==1
    tg3c_tsv_eval_r7 % call TSV code
    break
end
disp(' ')

%%%%%%%%%% continuous menu loop %%%%%%%%%%

while 1  % continuous loop that can only be exited by menu option #7

    % print out the options menu
    disp('Main Menu for 802.15.3c SV Channel Model ...')
    disp(' Option 1: Analyze Statistics of a Previously Generated CM Impulse Response & View Realizations')
    disp(' Option 2: Generate CM Impulse Responses by Appling Spatial Filtering & Entering Sample Rate')
    disp(' Option 3: Obtain Cluster Statistics')
    disp(' Option 4: Graphically View S-V Clusters for a Particular CM')
    disp(' Option 5: Generate All New S-V Clusters')
    disp(' Option 6: Load S-V Parameters and Make Directories')
    disp(' Option 7: Exit Program')
    disp(' Option 8: Revision History')
    pause(0.8); disp(' ')
    
    % input the menu option number
    OptNum = -1; % initialize to illegal value
    while (OptNum>8 || OptNum<1)
        OptNum = input('Input Menu Option Number [1, 2, 3, 4, 5, 6, 7, 8] ');
    end

    % some menu options require inputting a CM number ... get it now
    if OptNum==1 || OptNum==2 || OptNum==3 || OptNum==4
        disp(' ')
        disp('Please Enter Channel Model Number of Interest')
        ChanNum = -1; % initialize to illegal value
        while (ChanNum>10 || ChanNum<0)
            ChanNum = input('Please Input SV CM Number:(1.5, 2.5, 5, 6, 9.3, 10) ');
        end
        if ChanNum == 0
            ChanNum = -1; % initialize to illegal value
            while (ChanNum>10 || ChanNum<0)
                disp('Legacy Menu - not for use by 803.15.3c - Please Input CM Number:')
                ChanNum = input('(1.1, 1.2, 1.3, 1.4, 1.5, 2.1, 2.2, 2.3, 2.4, 2.5, 3.1, 3.2, 4.1, 4.2, 4.3, 5, 6, 7, 8, 9.1, 9.2, 9.3, 10) ');
            end
        end
        
        save ChanNum ChanNum

    end

    switch OptNum % option switch 
    
        case(1)
            disp(' ');disp('Warning: you must generate an impulse response, via menu option 2, before you can analyze it.')
            disp('If the program crashes at this point, it is probably because the impulse response has not been generated.')
            AnalyzeImpulse; % call Analyze Impulse
        
        case(2)
            GenImpulse; % call Generate Impulse
        
        case(3)
            ClusterStats; % call Cluster Statistics

        case(4)
            ViewClusters; % call View S-V Clusters
        
        case(5)
            % play a gong to sound a warning
            load xgong
            p=audioplayer(y, Fs);
            play(p)

            % get response to prompt for overwriting files
            display('  Caution: proceeding will overwrite previously stored clusters!')
            CautionQuestion = -1; % initialize to illegal value
            while (CautionQuestion>2 || CautionQuestion<1)
                CautionQuestion = input('   Do you want to proceed? [1="yes", 2="no"] ');
			end

			GoldenQuestion = -1; % initialize to illegal value, used next
            switch CautionQuestion % action due to overwriting prompt
                case(1)
                    % get response to prompt for generating golden clusters
                    while (GoldenQuestion>2 || GoldenQuestion<1)
                        GoldenQuestion = input('   Do you want to regenerate "Golden Clusters"? [1="yes", 2="no"] ');
                    end
                    save GoldenQuestion GoldenQuestion
                    LoadParams; % first load and store the parameters as defined by TG3c
                    GenClusters; % call Generate Clusters
                case(2)
                    display('   *** Terminating Program ***')
            end

        case(6)
            LoadParams; % call Load S-V Parameters
            disp('*** MAKING DIRECTORIES - THEY MAY ALREADY EXIST ***')
        
            % directory list ...
            % this list needs to reflect the CM numbers shown above
            mkdir('CM1.1'); mkdir('CM1.2'); mkdir('CM1.3'); mkdir('CM1.4'); mkdir('CM1.5')
            mkdir('CM2.1'); mkdir('CM2.2'); mkdir('CM2.3'); mkdir('CM2.4'); mkdir('CM2.5')
            mkdir('CM3.1'); mkdir('CM3.2'); mkdir('CM4.1'); mkdir('CM4.2');mkdir('CM4.3')
            mkdir('CM5'); mkdir('CM6'); mkdir('CM7'); mkdir('CM8'); mkdir('CM9.1')
            mkdir('CM9.2'); mkdir('CM9.3'); mkdir('CM10')
            
        case(7) % exiting the continuous loop
            % play a little joyful music now that we are done
            load xhandel;
            p = audioplayer(y, Fs); 
            play(p, [1 (get(p, 'SampleRate') * 4.5)]);
            break % go to the next statement past the continuous loop
            
        case(8)
            disp(' '); disp(' Revision History')
            disp('rev 1.01, 9 Jan 07 - GenClusters corrected to generate CM9.3 and CM10')

    end

    % run a quick test to see if we properly generated Golden Clusters
    if OptNum==5 && GoldenQuestion==1
        TestGoldenCluster
    end
    
    % bottom of continuous loop ... display "strike any key" message
    beep
    disp(' '); disp('*** Strike Any Key to Continue ***')
    pause % input a key stroke to continue
    clear all; close all; clc; beep % clear the screen to start thru menu again

end

% all done
disp(' ');disp('ALL DONE'); disp(' '); beep