%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% S-V Engine
% 20 December 2006, Release 1.0
%
% Comments: This function does the work of generating the S-V clusters
%
% Input:
%
% Output:
%
% Internal Variables:
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% note: arguments n, PLo and LowSigma_s are not currently used
function sv(n, PLo, LowSigma_s, CapLambda, ... 
  LowLambda, CapGamma, LowGamma, LowSigma_c, LowSigma_r, LowSigma_phi, ... 
  L_mean, Delta_CapK, Delta_LowK, nlos, TSV, SynNLOS, ChanNum, NumChannels)

% here is how this works ... allowed states
% - NLOS: nlos=1, TSV=0, SynNLOS=0
% - Regular LOS: nlos=0, TSV=0, SynNLOS=0
% - TSV LOS: nlos=0, TSV=1, SynNLOS=0
% - Synthesize NLOS from LOS: nlos=0, TSV=0, SynNLOS=1

% display state of the SV engine
if nlos==1&&TSV==0&&SynNLOS==0
    disp('  --> using NLOS SV engine ')
elseif nlos==0&&TSV==0&&SynNLOS==0
    disp('  --> using Regular LOS SV engine ')
elseif nlos==0&&TSV==1&&SynNLOS==0
    disp('  --> using TSV LOS SV engine ')
elseif nlos==0&&TSV==0&&SynNLOS==1
    disp('  --> using synthesized NLOS SV engine ')
else
    disp('  --> illegal SV model state ')
    halt
end

%%%%% translate my storage names to the names used in contribution 15-06-0302-02-003c %%%%%
% note: the S-V engine was extracted from this contribution
K_Los_dB = Delta_CapK;                          % LOS K factor
Kc_dB = Delta_LowK;                             % Cluster K factor
Sigma1 = LowSigma_c;                            % Cluser log-normal [dB]
Sigma2 = LowSigma_r;                            % Ray log-normal [dB]
GammaRay = LowGamma;                            % Ray decay
GammaCluster = CapGamma;                        % Cluster decay
LambdaCluster = CapLambda;                      % Poisson cluster arrival rate
LambdaRay = LowLambda;                          % Poisson ray arrival rate
Deg = pi/180;                                   % [Deg] <-> [Rad]
K_Los = 10^(K_Los_dB/10);Kc = 10^(Kc_dB/10);    % Convert from [dB]   
RayAS = LowSigma_phi*Deg;                       % Deg -> Rad
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% determine the number of clusters
if TSV==1 % we need to trap the TSV case
    NumClusters=-1;
    while NumClusters<2 % TSV realizations need at least 2 clusters
        NumClusters = max(1, poissrnd(L_mean)); % number of clusters (at least 2 cluster)
    end
else
    NumClusters = max(1, poissrnd(L_mean)); % number of clusters (at least 1 cluster)
end

NumRaysPerCluster = ceil(5*GammaRay*LambdaRay); % Number of rays per cluster

% Initialize some arrays
ToaCluster=zeros(NumClusters,NumChannels);
AoaCluster=zeros(NumClusters,NumChannels);
AvePowCluster=zeros(NumClusters,NumChannels);
InsPowCluster=zeros(NumClusters,NumChannels);

% Initialize and set LOS parameters
ToaRay = zeros(NumRaysPerCluster, NumClusters, NumChannels); % Initialize matrix
AoaRay = zeros(NumRaysPerCluster, NumClusters, NumChannels); % Initialize matrix
InsPowRay = zeros(NumRaysPerCluster, NumClusters, NumChannels); % Initialize matrix
AvePowRay = zeros(NumRaysPerCluster, NumClusters, NumChannels); % Initialize matrix

for ChanIndex = 1:NumChannels % loop over number of channels

    % Cluster Arrays for this channel
    ToaCluster(:,ChanIndex) = poisson(LambdaCluster, NumClusters);                 % Cluster Time of Arrival [ns]
    AoaCluster(:,ChanIndex) = 2*pi*rand(1, NumClusters);                           % Cluster Angle of Arrival [rad]
    AvePowCluster(:,ChanIndex) = exp(-ToaCluster(:,ChanIndex)/GammaCluster)/K_Los; % Cluster exponential APDP
    InsPowCluster(:,ChanIndex) = log_normal(AvePowCluster(:,ChanIndex), Sigma1);   % Cluster instantaneous power

    % Ray Arrays for this channel
    for ClusterIndex = 1:NumClusters % the first cluster is the LOS
        if nlos==1&&TSV==0&&SynNLOS==0 % NLOS
            ToaRay(:, ClusterIndex, ChanIndex)=poisson(LambdaRay, NumRaysPerCluster);  % Intra-cluster ray Time of Arrival [ns]
            AoaRay(:, ClusterIndex, ChanIndex) = RayAS*randn(NumRaysPerCluster, 1);    % Intra-cluster ray Angle of Arrival [deg]
            AvePowRay(:,ClusterIndex,ChanIndex) = Kc*InsPowCluster...
                (ClusterIndex,ChanIndex)*exp(-ToaRay(:, ClusterIndex, ChanIndex)/GammaRay);  % Intra-Cluster APDP
            InsPowRay(:, ClusterIndex, ChanIndex) = log_normal(AvePowRay...
                (:,ClusterIndex,ChanIndex), Sigma2);  % Ray instantaneous power
        elseif nlos==0&&TSV==0&&SynNLOS==0 % Reg. LOS
            if ClusterIndex==1 % identify the first cluster
                ToaCluster(ClusterIndex, ChanIndex)=0;    % set TOA to zero
                AoaCluster(ClusterIndex, ChanIndex)=pi;   % angel of arrival at pi
                AvePowCluster(ClusterIndex, ChanIndex)=1; % average power of cluster to 1
                InsPowCluster(ClusterIndex, ChanIndex)=1; % instantaneous power of cluster to 1
            end

            ToaRay(:, ClusterIndex, ChanIndex)=poisson(LambdaRay, NumRaysPerCluster);  % Intra-cluster ray Time of Arrival [ns]
            AoaRay(:, ClusterIndex, ChanIndex) = RayAS*randn(NumRaysPerCluster, 1);    % Intra-cluster ray Angle of Arrival [deg]
            AvePowRay(:,ClusterIndex,ChanIndex) = Kc*InsPowCluster(ClusterIndex,...
                ChanIndex)*exp(-ToaRay(:, ClusterIndex, ChanIndex)/GammaRay);  % Intra-Cluster APDP
            InsPowRay(:, ClusterIndex, ChanIndex) = log_normal(AvePowRay(:,...
                ClusterIndex,ChanIndex), Sigma2);  % Ray instantaneous power

            ToaRay(1, 1, ChanIndex)= 0;  % Intra-cluster ray Time of Arrival [ns]
            AoaRay(1, 1, ChanIndex) = 0;    % Intra-cluster ray Angle of Arrival [deg]
            AvePowRay(1, 1,ChanIndex) = 1;  % Intra-Cluster APDP
            InsPowRay(1, 1, ChanIndex) = 1;  % Ray instantaneous power

        elseif nlos==0&&TSV==1&&SynNLOS==0 % TSV LOS
            if ClusterIndex==1
                ToaCluster(ClusterIndex, ChanIndex)=0;
                AoaCluster(ClusterIndex, ChanIndex)=pi;
                AvePowCluster(ClusterIndex, ChanIndex)=1;
                InsPowCluster(ClusterIndex, ChanIndex)=1;
                
                ToaRay(1, ClusterIndex, ChanIndex)= 0;  % Intra-cluster ray Time of Arrival [ns]
                AoaRay(1, ClusterIndex, ChanIndex) = 0;    % Intra-cluster ray Angle of Arrival [deg]
                AvePowRay(1,ClusterIndex,ChanIndex) = 1;  % Intra-Cluster APDP
                InsPowRay(1, ClusterIndex, ChanIndex) = 1;  % Ray instantaneous power
            else
                ToaRay(:, ClusterIndex, ChanIndex)=poisson(LambdaRay, NumRaysPerCluster);  % Intra-cluster ray Time of Arrival [ns]
                AoaRay(:, ClusterIndex, ChanIndex) = RayAS*randn(NumRaysPerCluster, 1);    % Intra-cluster ray Angle of Arrival [deg]
                AvePowRay(:,ClusterIndex,ChanIndex) = Kc*InsPowCluster(ClusterIndex,...
                    ChanIndex)*exp(-ToaRay(:, ClusterIndex, ChanIndex)/GammaRay);  % Intra-Cluster APDP
                InsPowRay(:, ClusterIndex, ChanIndex) = log_normal(AvePowRay(:,...
                    ClusterIndex,ChanIndex), Sigma2);  % Ray instantaneous power
            end 
        elseif nlos==0&&TSV==0&&SynNLOS==1 % Synth NLOS
            if ClusterIndex==1
                % zero out the if cluster, which is the LOS cluster
                ToaCluster(ClusterIndex, ChanIndex)=0;
                AoaCluster(ClusterIndex, ChanIndex)=0;
                AvePowCluster(ClusterIndex, ChanIndex)=0;
                InsPowCluster(ClusterIndex, ChanIndex)=0;
                
                ToaRay(1, ClusterIndex, ChanIndex)= 0;  % Intra-cluster ray Time of Arrival [ns]
                AoaRay(1, ClusterIndex, ChanIndex) = 0;    % Intra-cluster ray Angle of Arrival [deg]
                AvePowRay(1,ClusterIndex,ChanIndex) = 1;  % Intra-Cluster APDP
                InsPowRay(1, ClusterIndex, ChanIndex) = 0;  % Ray instantaneous power
            else
                ToaRay(:, ClusterIndex, ChanIndex)=poisson(LambdaRay, NumRaysPerCluster);  % Intra-cluster ray Time of Arrival [ns]
                AoaRay(:, ClusterIndex, ChanIndex) = RayAS*randn(NumRaysPerCluster, 1);    % Intra-cluster ray Angle of Arrival [deg]
                AvePowRay(:,ClusterIndex,ChanIndex) = Kc*InsPowCluster(ClusterIndex,...
                    ChanIndex)*exp(-ToaRay(:, ClusterIndex, ChanIndex)/GammaRay);  % Intra-Cluster APDP
                InsPowRay(:, ClusterIndex, ChanIndex) = log_normal(AvePowRay(:,...
                    ClusterIndex,ChanIndex), Sigma2);  % Ray instantaneous power
            end 
        end
    end
end

% put together all the cluster ray arrays
NumRays=NumClusters*NumRaysPerCluster;

% initialize needed arrays
ToaRayArray=zeros(NumRays,NumClusters,NumChannels);
AoaRayArray=zeros(NumRays,NumClusters,NumChannels);
AvePowRayArray=zeros(NumRays,NumClusters,NumChannels);
InsPowRayArray=zeros(NumRays,NumClusters,NumChannels);

% put ray clusters in a rank column order
for ChanIndex=1:NumChannels
    for ClusterIndex=1:NumClusters
        ToaRayArray(1+(ClusterIndex-1)*NumRaysPerCluster:NumRaysPerCluster+...
            (ClusterIndex-1)*NumRaysPerCluster,ClusterIndex,ChanIndex)=...
            ToaRay(1:NumRaysPerCluster,ClusterIndex,ChanIndex);
        AoaRayArray(1+(ClusterIndex-1)*NumRaysPerCluster:NumRaysPerCluster+...
            (ClusterIndex-1)*NumRaysPerCluster,ClusterIndex,ChanIndex)=...
            AoaRay(1:NumRaysPerCluster,ClusterIndex,ChanIndex);
        AvePowRayArray(1+(ClusterIndex-1)*NumRaysPerCluster:NumRaysPerCluster+...
            (ClusterIndex-1)*NumRaysPerCluster,ClusterIndex,ChanIndex)=...
            AvePowRay(1:NumRaysPerCluster,ClusterIndex,ChanIndex);
        InsPowRayArray(1+(ClusterIndex-1)*NumRaysPerCluster:NumRaysPerCluster+...
            (ClusterIndex-1)*NumRaysPerCluster,ClusterIndex,ChanIndex)=...
            InsPowRay(1:NumRaysPerCluster,ClusterIndex,ChanIndex);
    end
end

% save off resulting arrays
cd(strcat('CM',num2str(ChanNum)))
save RayArray ToaRayArray AoaRayArray AvePowRayArray InsPowRayArray
cd ..
clear ToaRayArray AoaRayArray AvePowRayArray InsPowRayArray 

% now put together all the cluster ray arrays
Toa=zeros(NumRaysPerCluster,NumClusters,NumChannels);
Aoa=zeros(NumRaysPerCluster,NumClusters,NumChannels);
InsPow=zeros(NumRaysPerCluster,NumClusters,NumChannels);

% this is where we add the cluster info with the ray info
for ChanIndex=1:NumChannels
    for ClusterIndex=1:NumClusters
        Toa(:,ClusterIndex,ChanIndex)=ToaCluster(ClusterIndex,ChanIndex)+...
            ToaRay(:,ClusterIndex,ChanIndex);
        Aoa(:,ClusterIndex,ChanIndex)=AoaCluster(ClusterIndex,ChanIndex)+...
            AoaRay(:,ClusterIndex,ChanIndex)-mean(AoaRay(:,ClusterIndex,ChanIndex))/2;
        InsPow(:,ClusterIndex,ChanIndex)=InsPowRay(:,ClusterIndex,ChanIndex); % redundant but harmless
    end
end

% save off the resulting arrays
cd(strcat('CM',num2str(ChanNum)))
save ClusterInfo ToaCluster AoaCluster AvePowCluster InsPowCluster
save RayInfo ToaRay AoaRay AvePowRay InsPowRay
cd ..
clear ToaCluster AoaCluster AvePowCluster InsPowCluster ToaRay AoaRay AvePowRay InsPowRay

% prepare the arrays for the ray+clusters
ToaArray=zeros(NumRays,NumClusters,NumChannels);
AoaArray=zeros(NumRays,NumClusters,NumChannels);
InsPowArray=zeros(NumRays,NumClusters,NumChannels);

% put ray+clusters in a rank column order
for ChanIndex=1:NumChannels
    for ClusterIndex=1:NumClusters
        ToaArray(1+(ClusterIndex-1)*NumRaysPerCluster:NumRaysPerCluster+...
            (ClusterIndex-1)*NumRaysPerCluster,ClusterIndex,ChanIndex)=...
            Toa(1:NumRaysPerCluster,ClusterIndex,ChanIndex);
        AoaArray(1+(ClusterIndex-1)*NumRaysPerCluster:NumRaysPerCluster+...
            (ClusterIndex-1)*NumRaysPerCluster,ClusterIndex,ChanIndex)=...
            Aoa(1:NumRaysPerCluster,ClusterIndex,ChanIndex);
        InsPowArray(1+(ClusterIndex-1)*NumRaysPerCluster:NumRaysPerCluster+...
            (ClusterIndex-1)*NumRaysPerCluster,ClusterIndex,ChanIndex)=...
            InsPow(1:NumRaysPerCluster,ClusterIndex,ChanIndex);
    end
end

% save off the resulting arrays
cd(strcat('CM',num2str(ChanNum)))
save FullValues Toa Aoa InsPow
cd ..
clear Toa Aoa InsPow 

% put together time base and impulse as a column vector
ToaVector=zeros(NumRays,NumChannels);
AoaVector=zeros(NumRays,NumChannels);
InsPowVector=zeros(NumRays,NumChannels);

for ChanIndex=1:NumChannels
    if NumClusters>1
        ToaVector(:,ChanIndex)=sum(ToaArray(:,:,ChanIndex)')';
        AoaVector(:,ChanIndex)=sum(AoaArray(:,:,ChanIndex)')';
        InsPowVector(:,ChanIndex)=sum(InsPowArray(:,:,ChanIndex)')';
    else
        ToaVector(:,ChanIndex)=ToaArray(:,1,ChanIndex);
        AoaVector(:,ChanIndex)=AoaArray(:,1,ChanIndex);
        InsPowVector(:,ChanIndex)=InsPowArray(:,1,ChanIndex);
    end
end

% take care of the 2*pi wrap around
AoaWrappedVector=mod(AoaVector,2*pi)-pi; % recenter and pi wrap the AoA vector

% save off the resulting arrays
cd(strcat('CM',num2str(ChanNum)))
save FullArray ToaArray AoaArray InsPowArray
cd ..
clear ToaArray AoaArray InsPowArray 

% derive the impulse response
SortedTime=zeros(size(ToaVector));
SortedIndex=zeros(size(ToaVector));
SortedAmp=zeros(size(InsPowVector));
SortedAng=zeros(size(AoaWrappedVector));

for ChanIndex=1:NumChannels
    [SortedTime(:,ChanIndex),SortedIndex] = sort(ToaVector(:,ChanIndex));    % sort in ascending time order
    SortedAmp(:,ChanIndex)=InsPowVector(SortedIndex,ChanIndex); % use the sorted index to sort the amplitude
    SortedAng(:,ChanIndex)=AoaWrappedVector(SortedIndex,ChanIndex); % use the sorted index to sort the angle
end

% save off the resulting vectors
cd(strcat('CM',num2str(ChanNum)))
save FullVectors ToaVector InsPowVector AoaWrappedVector
save SortedVectors SortedAmp SortedTime SortedAng
cd ..
clear ToaVector InsPowVector AoaWrappedVector
clear SortedAmp SortedTime SortedAng

% save off the useful information that will be needed later
t0=zeros(1,NumChannels); % dummy place holder t0 for the LOS arrival time
cd(strcat('CM',num2str(ChanNum)))
save ImpInfoStuff t0 NumRays NumRaysPerCluster NumClusters NumChannels
cd ..

return