%Main Script M-file
%----------------------------------------------------------------------------------
%
% Description:
% This function M-file generates sets of AOA, TOA, and amplitude of rays in 
%continuous time on the basis of TSV model, and also generates and evaluates 
%discrete impulse responses, which are generated using the sets of the AOA, 
%TOA and amplitude of rays in the continuous time.

% IEEE802.15.4a distribution code was modified by
% H. Harada, R. Funada, H. Sawada, Y. Shoji, C-S Choi, and S. Kato
%----------------------------------------------------------------------------------

%***************************** Preparation Part ****************************
clear variables
close all
randn_seed_ch=12;
rand_seed_ch=12;
randn('state',randn_seed_ch);	% Initialize state of function for repeatability
rand('state',rand_seed_ch);     % Initialize state of function for repeatability

%***************************** Parameter Setup *****************************
tg3c_tsv_menu_disp

%************************* Load Channel Parameters *************************
[adist, nlos, los_beta_flg, Omega0, smallk, Lmean, Lam, lambda, Gam, gamma,...
	std_ln_1,std_ln_2, sigma_fai, L_pl, tx_hpbw, rx_hpbw0]= tg3c_tsv_params_r4(cm_num, fc);

%************************* Display Model Parameters ************************
disp(' ')
fprintf(1,['***************************** Model Parameters ****************************\n' ...
	'  Channel Model Index = CM%d.%d\n' ...
	'  Number of Channel Realizations = %d\n' ...
	'  Distance between Tx and Rx = %d[m]\n' ...
	'  LOS Component Pathloss = %.4f[dB]\n' ...
	'  Lmean = %d, Omega0  = %5.5f[dB], smallk= %5.5f, sigma_fai = %.4f[deg]\n' ...
	'  1/Lam = %.4f, 1/lambda = %.4f, Gam = %.4f, gamma = %.4f\n' ...
	'  std_ln_1 = %.4f, std_ln_2 = %.4f\n'...
	'***************************************************************************\n'],...
	channel_num,channel_num2, num_channels, adist, L_pl, Lmean, Omega0, smallk,...
	sigma_fai, 1/Lam, 1/lambda, Gam, gamma, std_ln_1, std_ln_2)

if op_num == 2  || op_num == 3 || channel_num == 7 || channel_num == 8
	fprintf(1,['  Antenna Model used in Simulation = %d\n' ...
		'  Measured Antenna Model used in simulation:\n'...
		'  Beam-width of Measured Tx Antenna = %3d[deg]\n'...
		'  Beam-width of Measured Rx Antenna = %3d[deg]\n' ...
		'  Beam-width of Simulated Rx Antenna = %3d[deg]\n'],...
	ant_type,tx_hpbw, rx_hpbw0,rx_hpbw);
end

%****************** Generate Continuous Impulse Responses ******************
% tg3c_tsv_ct_r5.m
% Output values
%	h_ct	: Impulse responses (before convolution with antenna gain ) in continuous time
%	t_ct	: TOAs of h
%	t0		: Arrival time of the first ray of the first SV cluster
%	np		: Number of paths in clusters including LOS component
% Output values added for making TG3c channel model
%	h2_ct	: Impulse responses after convolution with antenna gain in continuous time
%	beta	: Amplitude of the LOS component
%	aoa_ct	: AOAs of rays in clusters including LOS component in continuous time 

[beta,h_ct,h2_ct,aoa_ct, t_ct,t0,np,cl_idx] = tg3c_tsv_ct_r6(...
	nlos, num_channels,...									% Channel params
	adist, fc, los_beta_flg, L_pl, ...						% T-S-V model params
	Lam, lambda, Gam, gamma, std_ln_1, std_ln_2, Lmean, ...	% SV model params
	Omega0, smallk, sigma_fai,...							% SV model params regarding TSV model
	tx_hpbw, rx_hpbw, ant_type,...							% Antenna model params used in simulation
	op_num);

if op_num == 1
	if MAT_flag == 1
		MAT_NAME=strcat('CM',num2str(cm_num),'TsvGoldSet');
		save(MAT_NAME,'cm_num','np','h_ct','t_ct','aoa_ct');
		fprintf('\nMAT File Including Continuous Impulse Responses has been successfully created\n\n')
		return;
	end
else
	%****************************** Phase rotation *****************************
	if nlos == 0
		h2_ct(2:end,:) = exp(j*2*pi*rand(size(h2_ct(2:end,:)))).*h2_ct(2:end,:);
	else
		h2_ct = exp(j*2*pi*rand(size(h2_ct))).*h2_ct;
	end
	%********* Convert to Continuous Impulse Responses to Discrete Ones ********
	[hN,N] = tg3c_sv_cnvrt_ct( h2_ct, t_ct, np, num_channels, ts);

	% If we wanted complex baseband model or to impose some filtering function,
	% this would be a good place to do it
	if N > 1,
		h = resample(hN, 1, N);  % decimate the columns of hN by factor N
	else
		h = hN;
	end
	% correct for 1/N scaling imposed by decimation

	h = h * N;

	%******************** Compute Some of Highlight Results ********************
	tg3c_tsv_results_disp
	
	%******************* Save Some Parameters to the MAT file ******************
	if MAT_flag2 == 1		
		MAT_NAME2=strcat('tsv_dIR_cm',num2str(cm_num),'_n',num2str(num_channels),...
			'_at',num2str(ant_type),'_fs',(num2str(ceil(fs))));
		%save(MAT_NAME2,'cm_num','num_channels','h_ct','t_ct','aoa_ct','h','ant_type','Kf','RMS_delay','fs','rand_seed_ch','randn_seed_ch');
		save(MAT_NAME2,'cm_num','num_channels','h','ant_type','Kf','RMS_delay','fs','rand_seed_ch','randn_seed_ch');

		fprintf('\nMAT File has been successfully created\n\n')
	end	
end
