%----------------------------------------------------------------------------------
%
% Description:
% This Script M-file setups input arguments
% Programmed by R. Funada and H. Harada
%----------------------------------------------------------------------------------

disp(' ')
disp('Main Menu for 802.15.3c TSV Channel Model ...')
disp(' Option 1: Create CM Golden Set (Power,TOA, and AOA of Rays before Antenna Gain Convolution)')
disp(' Option 2: Create Discrete CM Impulse Responses using Antenna Model')
disp(' Option 3: Create Discrete CM Impulse Responses using Antenna Model with Simulation Results Displayed')
disp(' Option 4: Exit Program')
op_num=input('\nSelect menu index: Option ');
if (op_num ~= 1 && op_num ~= 2 && op_num ~= 3); disp('Exit from This Program'); return; end

%*************************** Determine Parameters **************************
% Set channel model index (cm_num)
disp(' ')
disp('******************* T-S-V Channel Model Parameter Setup *******************')
disp('Channel Model Index: ')
disp('		Scenario	Environment	Engine')
disp('CM1:	LOS			Residential	TSV Engine')
disp('CM2:	NLOS		Residential	TSV Engine (LOS component exstraction mode)')
disp('CM3:	LOS			Office		TSV Engine')
disp('CM4:	NLOS		Office		TSV Engine')
disp('CM5:	LOS			Library		SV engine')
disp('CM6:	NLOS		Library		SV engine')
disp('CM7:	LOS			Desktop		TSV Engine')
disp('CM8:	NLOS		Desktop		TSV engine (LOS component exstraction mode)')
disp('CM9:	LOS			Kiosk		TSV engine (LOS component exstraction mode)')
channel_num=input('\nSelect Channel Model index to Generate: CM');
if isempty(channel_num) == 1,channel_num = 1;end % added by Nishiguchi (26-Jan-2007)

disp(' ')
switch channel_num
	case 1
		disp('-------- LOS Residential Channels will be Generated with TSV Engine -------')
		disp('Measured Antenna model used in simulation configulation')
		disp('CM1.1: Tx : 360 deg, Rx : 15 deg provided by NICT')
		disp('CM1.2: Tx : 60  deg, Rx : 15 deg')
		disp('CM1.3: Tx : 30  deg, Rx : 15 deg')
		disp('CM1.4: Tx : 15  deg, Rx : 15 deg')
		disp('CM1.5: Tx : 360 deg, Rx : 15 deg provided by NICTA')
	case 2
		disp('------- NLOS Residential Channels will be Generated with TSV Engine -------')
		disp('LOS component exstraction mode')
		disp('Measured Antenna model used in simulation configulation')
		disp('CM2.1: Tx : 360 deg, Rx : 15 deg')
		disp('CM2.2: Tx : 60  deg, Rx : 15 deg')
		disp('CM2.3: Tx : 30  deg, Rx : 15 deg')
		disp('CM2.4: Tx : 15  deg, Rx : 15 deg')
	case 3
		disp('---------- LOS office Channels will be Generated with TSV Engine ----------')
		disp('Measured Antenna model used in simulation configulation')
		disp('CM3.1: Tx : 30 deg, Rx : 30 deg')
		disp('CM3.2: Tx : 60 deg, Rx : 60 deg')
	case 4
		disp('---------- NLOS office Channels will be Generated with TSV Engine ---------')
		disp('Measured Antenna model used in simulation configulation')
		disp('CM4.1: Tx : 360 deg, Rx : 15 deg')
		disp('CM4.2: Tx : 30 deg, Rx : 15 deg')
	case 7
		disp('---------- LOS Desktop Channels will be Generated with TSV Engine ---------')
		disp('Measured Antenna Model used in Simulation Configulation')
		disp('CM7.1: Tx : 30 deg, Rx : 30 deg')
		disp('CM7.2: Tx : 60 deg, Rx : 60 deg')
	case 8
		disp('--------- NLOS Desktop Channels will be Generated with TSV Engine ---------')
		disp('LOS component exstraction mode')
		disp('Measured Antenna Model used in Simulation Configulation')
		disp('CM8.1: Tx : 30 deg, Rx : 30 deg')
		disp('CM8.2: Tx : 60 deg, Rx : 60 deg')
	case 9
		disp('---------- LOS Kiosk Channels will be Generated with TSV Engine ---------')
		disp('Measured Antenna Model used in Simulation Configulation')
		disp('CM9.1: Tx : 30 deg, Rx : 30 deg, Environment 1')
		disp('CM9.2: Tx : 30 deg, Rx : 30 deg, Environment 2')
	otherwise
		disp('Since CM based on SV engine was selected, exit from this MATLAB Code ')
		return
end

disp('NOTICE: Rx Antenna Beam-width can be changed, whereas Tx Antenna Beam-width is fixed, ')
channel_num2=input(strcat('\nSelect CM to Generate: CM',num2str(channel_num),'.'));
if isempty(channel_num2) == 1,channel_num2 = 1;end % added by Nishiguchi (26-Jan-2007)
cm_num=10*channel_num+channel_num2;

disp('------------ Center Carrier Frequency ------------')
disp('Set Center Carrier Frequency in GHz :');
disp('If skipped, this variable will be set to be 60 (GHz)')
fc0 = input('->');
if isempty(fc0) == 1,fc0 = 60;end
fc=fc0*1e9;

disp(' ')
disp('--------------- Channel Realization --------------')
disp('Set Number of Channel Realizations');
disp('If skipped, Number of Channel Realizations will be set to be 100')
num_channels = input('->');
if isempty(num_channels) == 1,num_channels = 100;end

if op_num == 2 || op_num == 3 || channel_num == 7

	disp(' ')
	disp('------------------ Antenna Model -----------------')
	disp('Antenna Model: ')
	disp('Model 1: Reference Antenna Model')
	disp('Model 2: Gaussian-Distributed Antenna Model')
	disp('Set Antenna Model Used in Simulation: Model ');
	disp('If skipped, Reference Antenna Model will be used')
	ant_type=input('->');
	if isempty(ant_type) == 1
		ant_type = 1;
	end


	disp(' ')
	disp('------------------ Rx antenna HPBW -----------------')
	disp('Input Rx Beam-width used in Simulation in Deg from 0 to 360 Deg')
	disp('If skipped, this variable is set to 30 (Deg) (Default value)')
	rx_hpbw=input('->');
	if isempty(rx_hpbw) == 1
		rx_hpbw = 30;
	end

	if op_num == 2 || op_num == 3

		disp('------------------ Sample rate -----------------')
		disp('Set Sample Rate in GHz :');
		disp('If skipped, this Variable is set to be 1 (GHz)')
		fs=input('->');
		if isempty(fs) == 1
			fs = 1;
		end
		ts=1./fs;
		disp('***************************** Save and Display ****************************')

		disp(' ')
		disp('--------- Save Discrete Impulse Responses --------')
		MAT_flag2 = input('Save Discrete Impulse Responses to MAT file? YES(1) or NO(0)  ->');
		if isempty(MAT_flag2) == 1,MAT_flag2 = 0;	end
		if op_num == 3
			disp('----------- Display Simulation Results -----------')
			disp_flg = input('Display Antenna Model used in Simulation? YES(1) or NO(0) ->');
			if isempty(disp_flg) == 1,disp_flg = 0;	end
			disp_flg1 = input('Display Power Delay Profile? YES(1) or NO(0) ->');
			if isempty(disp_flg1) == 1,disp_flg1 = 0; end
			if disp_flg1 ==1
				disp('2D Profile in Each Realization(1)')
				disp('2D Profile in All Realizations(2)')
				disp('3D Profile(3)')
				disp_flg2 = input('->');
			end
		end
	else
		MAT_flag = input('Save Goldenset (Power, TOA, AOA) and Some Parameters to MAT file? YES:1 NO:0  ->');
		if isempty(MAT_flag) == 1,MAT_flag = 0;	end
	end
else
	MAT_flag = input('Save Goldenset (Power, TOA, AOA) and Some Parameters to MAT file? YES:1 NO:0  ->');
	if isempty(MAT_flag) == 1,MAT_flag = 0;	end
	rx_hpbw = [];
	ant_type =[];
end