%----------------------------------------------------------------------------------
%
% Description:
% This Script M-file exhibits some of highlight simulation results
% IEEE802.15.4a distribution code was modified by
% R. Funada and H. Harada
%----------------------------------------------------------------------------------

h_len = size(h,1);
channel_energy = sum(abs(h).^2);

t = (0:(h_len-1)) * ts;  % for use in computing excess & RMS delays
excess_delay = zeros(1,num_channels);
RMS_delay = zeros(1,num_channels);
num_sig_paths = zeros(1,num_channels);
num_sig_e_paths = zeros(1,num_channels);
Kf=zeros(1,num_channels);

%------ Determine Excess Delay and RMS Delay ------
for k=1:num_channels
	sq_h = abs(h(:,k)).^2 / channel_energy(k);
	t_norm = t - t0(k);  % remove the randomized arrival time of first cluster
	excess_delay(k) = t_norm * sq_h;
	RMS_delay(k) = sqrt( ((t_norm-excess_delay(k)).^2) * sq_h );
	if nlos == 0
		Kf(k)= 10*log10(abs(h(1,k)).^2./sum(abs(h(2:end,k).^2)));
	else
		Kf(k)= NaN;
	end
end
fprintf(1,['\nAverage RMS_delay is %1.3f [ns] \n',...
	'Maximum RMS delay is %1.3f [ns] \n',...
	'Miminum RMS delay is %1.3f [ns] \n',...
	'Average Racian factor is %1.3f [dB]\n',...
	'Maximum Racian factor is %1.3f [dB] \n',...
	'Miminum Racian factor is %1.3f [dB] \n'],...
	mean(RMS_delay),max(RMS_delay),min(RMS_delay),mean(Kf),max(Kf),min(Kf))

if op_num == 3
	%----------- Display Antenna Models Used ----------
	if disp_flg == 1
		tsv_ant_gain_r6(ant_type,tx_hpbw,-180:180,[],1,'Tx Antenna Model used in Simulation');
		tsv_ant_gain_r6(ant_type,rx_hpbw,-180:180,[],2,'Rx Antenna Model used in Simulation');
	end

	%------------ Display Impulse Responses -----------
	if disp_flg1 == 1
		plot_color=['b-x'; 'r-x'; 'g-x'; 'y-x'; 'm-x';...
			'c-x'; 'g-x';'b-x'; 'r-x'; 'g-x'; 'y-x'; 'm-x'; 'c-x'; 'g-x'];
		h_len = size(h,1);
		t = (0:(h_len-1)) * ts;  % for use in computing excess & RMS delays

		figure(10);
		set(gca,'FontName','Arial','FontSize',12);
		xlabel('Time of arrival[ns]')
		ylabel('Average power [dB]')
		title('Power delay profile')
		hold on
		yy=10*log10(mean(abs(h).^2,2))+L_pl;
		plot(t,yy,'-x');
		grid on;
		sel=[10+L_pl max(yy)];
		axis([0 t(end) -60+L_pl max(sel)])

		if disp_flg2 ==1
			figure(11);
			set(gca,'FontName','Arial','FontSize',12);
			xlabel('Time of arrival[ns]')
			ylabel('Relative power [dB]')
			title('Power delay profile')
			grid on;hold on;
			for ij=1:num_channels
				COLOR0=rem(ij-1,7)+1;
				plot(t,20*log10(abs(h(:,ij)))+L_pl,plot_color(COLOR0,:));
				axis([0 t(end) -60+L_pl 10+L_pl])
				hold on;
				disp('Display the Next Channel Realization? Press Enter Key')
				skid=input('If Skip to Display the Next Result, Press "1", Otherwise Press Enter Key');
				if skid==1
					break;
				end
			end
		elseif disp_flg2 == 2
			figure(11);
			set(gca,'FontName','Arial','FontSize',12);
			xlabel('Time of arrival[ns]')
			ylabel('Relative power [dB]')
			title('Power delay profile')
			hold on
			yy=20*log10(abs(h))+L_pl;
			plot(t,yy,'-x');
			grid on;
			sel=[10+L_pl max(yy)];
			axis([0 t(end) -60+L_pl max(sel)])
		elseif disp_flg2 == 3
			for ij=1:num_channels
				figure(20);
				set(gca,'FontName','Arial','FontSize',12);
				xlabel('Time of arrival [ns])')
				ylabel('Angle of arrival [deg]')
				zlabel('10*log10(abs(amplitude)^2)+100 [dB]')
				title('Power delay profile')
				hold on

				COLOR0=rem(ij-1,7)+1;
				sv_power=20*log10(abs(h2_ct(1:np(ij),ij)))+100;
				ij2=find(sv_power>0);
				cl_idx2=cl_idx(ij2,ij);
				aoa_ct2=aoa_ct(ij2,ij);
				h_ct2=h_ct(ij2,ij);
				t_ct2=t_ct(ij2,ij);

				to=find(cl_idx2==1);
				stem3(t_ct2(to),aoa_ct2(to),20*log10(abs(h2_ct(to)))+100,'kx');
				hold on
				for il=2:max(cl_idx2)
					COLOR0=rem(il-1,7)+1;
					to=find(cl_idx2==il);
					stem3(t_ct2(to),aoa_ct2(to),20*log10(abs(h2_ct(to)))+100,plot_color(COLOR0,:));
					hold on
				end
				view([-40 50])
				grid on;
				disp('Display the Next Channel Realization? Press Enter Key')
				skid=input('If Skip to Display the Next Result, Press "1", Otherwise Press Enter Key');
				close(20)
				if skid==1
					break
				end
			end
		else
		end
	end
end