%----------------------------------------------------------------------------------
%
% Description :
% This function M-file computes amplitude of LOS component (beta) on the accordance 
% with the two-path theory of TSV model on the accordance with the two-path theory 
% of TSV model 
% Note: Antenna model in horizontal axis has the same gain as that in vertical axis
% Programmed by R. Funada, H. Sawada, Y. Shoji, and H. Harada 
%----------------------------------------------------------------------------------

function [beta] = tsv_beta_calc_r5(fc, muD, tx_hpbw, rx_hpbw, ant_type)

% Arguments:
%   fs			: Center carrier frequency
%   muD			: Average distance between TX and RX
%	tx_hpbw		: TX half-power angle in deg
%	rx_hpbw		: RX half-power angle in deg (horizontal and vartical gain are same)
%	ant_type	: Antenna model used in simulation

% Output values:
%	beta		: Amplitude of LOS component (beta)

%%%%%%%%%%%%%%%  Beta calculation (Monte carlo based)  %%%%%%%%%%%%%%%%%%%%

%	gamma0		: Reflection coefficient
gamma0	= 1;													% Assuming angle of incidence is large 

% this paramerres will be discussed
D0		= [-0.3 0.3]+muD;										% Range of D  (m)
Ht		= [0.0 0.30];											% Range of Ht (m)
Hr		= [0.0 0.30];											% Range of Hr (m)

% Determine TX and RX heights by the Monte-carlo method
h1	= (Ht(2)-Ht(1))*rand+Ht(1);
h2	= (Hr(2)-Hr(1))*rand+Hr(1);

% Determine distance between TX and RX by the Monte-carlo method
D	= (D0(2)-D0(1))*rand+D0(1);

% Wave length
ramda		= 3e8/fc;

%********** Calculate the reflection point of the relrection path **********
tx_p	= i.*h1;
rx_p	= D+i.*h2;
rfl_p	= D*h1/(h1+h2);

%************ Determene direction of direct and reflection paths ***********
tp		= angle([rx_p-tx_p (tx_p-rx_p) rfl_p-tx_p (rfl_p-rx_p)]);
tp		= tp./pi*180;

dr_theta	= tp(1);
dr_fai		= dr_theta;
rfl_theta	= tp(3);
rfl_fai		= -rfl_theta;

%******************** Determine Antenna horizontal gain ********************
% TX-------------------
% Direct path
gt1	= tsv_ant_gain_r6(ant_type, tx_hpbw, dr_theta);
% Reflection path
gt2	= tsv_ant_gain_r6(ant_type, tx_hpbw, rfl_theta);

% RX-------------------
% Direct path
gr1	= tsv_ant_gain_r6(ant_type, rx_hpbw, dr_fai);
% Reflection path
gr2	= tsv_ant_gain_r6(ant_type, rx_hpbw, rfl_fai);

beta	= (muD/D).*abs(gt1.*gr1+gt2.*gr2...
	.*gamma0.*exp(j.*(2*pi./ramda).*(2.*h1.*h2./D)));